c  Version 1.4 (NLS)
c**********************************************************************
c                       ===================
c                       SUBROUTINE : LBASIX
c                       ===================
c
c       This routine builds a list of the basis set indices in common 
c       block /indexl/ for the truncation parameters set in /eprdat/ 
c       or reads them from the index file "<name>.ind" which contains 
c       a pruned basis set produced by the EPRBL program. 
c
c       If one of the following conditions is true:
c         (1) The "new" flag is set nonzero on input
c         (2) the input filename is blank
c
c       this routine routine builds the full basis set indicated
c       by the truncation parameters specified in the MTS array.
c       These are as follows:
c          mts(1)  lemx    Maximum even L
c          mts(2)  lomx    Maximum odd L
c          mts(3)  kmn     Minimum K (K<0 => jK = -1)
c          mts(4)  kmx     Maximum K
c          mts(5)  mmn     Minimum M (M<0 => jM = -1)
c          mts(6)  mmx     Maximum M
c          mts(7)  ipnmx   Maximum pI
c          mts(8)  ldelta  L increment
c          mts(9)  kdelta  K increment
c          mts(10) ipsi0   Tilt angle flag
c          mts(11) in2     Nuclear spin
c          mts(12) jkmn    Minimum jK
c          mts(13) jmmn    Minimum jM
c
c       Uses:
c               ipar.f
c
c**********************************************************************
c
      subroutine lbasix( ixname,bss,mts,lthb,maxb,new,ierr )
      implicit none
c
      include 'nlsdim.inc'
      character*30 ixname,fname
      integer lthb,maxb,new,ierr,bss(11,lthb),mts(MXMTS)
c
      include 'eprprm.inc'
      include 'mtsdef.inc'
      include 'stdio.inc'
c
      include 'srsflg.inc'
      include 'srsset.inc'
c
      integer i,ioerr,j,
     #        kdel,ldel,lth,n,iparlr,krmn,mrmn,mts1(MXMTS)
c
      integer lr,mr,mrmx,kr,krmx,
     #        jr,nr,nrmn,nrmx,ir,irmx,
     #        ipsr,iqsr,ipnr,iqnr,
     #        ipnrmn,ipnrmx,iqnrmn,iqnrmx,
     #        ipsrmn,ipsrmx,iqsrmn,iqsrmx,
     #        imr,imrmn,imrmx,imstep,
     #        nrow
c
      logical fexist
      integer ipar,iroot
      external ipar,iroot
      common/imts/mts1 
c
c######################################################################
c
      ierr=0
      if (new.eq.0) then
c
c----------------------------------------------------------------------
c     Look for the basis set indices in the specified file
c----------------------------------------------------------------------
c
         fname=ixname
         lth=iroot(fname)
         fname=fname(:lth)//'.ind'
         lth=lth+4
         inquire(file=fname(:lth),exist=fexist)
         if (.not.fexist) then
c                                           *** file not found
            write(luout,1003) fname(:lth)
            if (luout.ne.luttyo) write (luttyo,1003) fname(:lth)
            ierr=-1
            return
         end if
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
         open (unit=ludisk,file=fname(:lth),status='old',
     #         access='sequential',form='unformatted',iostat=ioerr)
         read (ludisk,iostat=ioerr) lthb,(mts(i),i=1,NTRC)
         if (ioerr.ne.0 .or. lthb.gt.maxb) then
            if (ioerr.ne.0) then
c                                          *** error reading file
               write(luout,1002) fname(:lth)
               if (luout.ne.luttyo) write(luttyo,1002) fname(:lth)
            else
c                                           *** insufficient room 
               write(luout,1001) fname(:lth)
               if (luout.ne.luttyo) write(luttyo,1001) fname(:lth)
            end if
            ierr=-1
            return
         else
            read (ludisk,iostat=ioerr) ((bss(i,j),i=1,11),j=1,lthb)
         end if
c                                  *** error reading file
         if (ioerr.ne.0) then
            write(luout,1002) fname(:lth)
            if (luout.ne.luttyo) write(luttyo,1002) fname(:lth)
            ierr=-1
            return
         end if
c
         close(ludisk)
c
         do i=1,MXMTS
           mts1(i)=mts(i)
         enddo
c---trials
c      do j=1,lthb
c	write (4,1111) j,bss(1,j),bss(2,j),bss(3,j),bss(4,j),bss(5,j),
c     #	bss(6,j),bss(7,j),bss(8,j),bss(9,j),bss(10,j),bss(11,j)
c	enddo 
 1111 format(2x,11(2x,i2),2x,i2)
c
         return
      end if
c
c     mts(NKMN)=0.0D0
c     mts(NMMN)=0.0D0
c
c----------------------------------------------------------------------
c     Routine was called with blank filename or "new" flag nonzero: 
c     Construct a basis using specified MTS. First check the MTS
c----------------------------------------------------------------------
c
      if (mts(NLEMX).le.0 .or. mts(NLOMX).lt.0 .or.
     #    mts(NKMX).lt.0 .or.  mts(NMMX).lt.0 .or.
     #    mts(NIPNMX).lt.0 .or. mts(NKMN).gt.mts(NKMX) .or.
     #    mts(NMMN).gt.mts(NMMX) .or. mts(NJMX).lt.0 .or.
     #    mts(NNMX).lt.0 .or. mts(NIMX).lt.0 ) then
          write (luout,1004) (mts(i),i=1,NTRC)
         ierr=-1
         return
      end if
c
c----------------------------------------------------------------------
c     *** loop over lr ***
c----------------------------------------------------------------------
c
      do 100 lr=0,mts(NLEMX),1
        if((1-2*mod(abs(lr),2).ne.1).and.(lr.gt.mts(NLOMX))) go to 100
c       if(lr.eq.1) go to 100
c
c----------------------------------------------------------------------
c     *** loop over mr ***
c----------------------------------------------------------------------
c
       mrmx=min(mts(NMMX),lr)
c
       do 110 mr=0,mrmx,1
c
c----------------------------------------------------------------------
c     *** loop over kr ***
c----------------------------------------------------------------------
c
        krmx=min(mts(NKMX),lr)
c
        do 120 kr=0,krmx,kdelta
           if (kr.eq.0.and.ipar(lr).ne.1) go to 120
c
c----------------------------------------------------------------------
c     *** loop over jr ***
c----------------------------------------------------------------------
c
         do 122 jr=0,mts(NJMX),jdelta
c
c----------------------------------------------------------------------
c     *** loop over nr ***
c----------------------------------------------------------------------
c
          nrmx=min(mts(NNMX),jr)
c
          if (mr.eq.0) then
             nrmn=0
          else
             nrmn=-nrmx
          end if
          do 124 nr=nrmn,nrmx,1
c
c----------------------------------------------------------------------
c     *** loop over ir ***
c----------------------------------------------------------------------
c
          irmx=min(mts(NIMX),jr)
c
          do 125 ir=0,irmx
           if (ir.eq.0.and.ipar(jr).ne.1) go to 125
c
c----------------------------------------------------------------------
c     *** loop over ipnr ***
c----------------------------------------------------------------------
c
           ipnrmx=min(mts(NIPNMX),in2)
           if (mr.eq.0.and.nr.eq.0) then
              ipnrmn=0
           else
              ipnrmn=-ipnrmx
           end if
           do 126 ipnr=ipnrmn,ipnrmx,1
              if (psi.eq.0.0D0.and.(mr+nr).ne.ipnr) go to 126
c
c----------------------------------------------------------------------
c     *** loop over im ***
c----------------------------------------------------------------------
c
              imrmx=1
              if (hfdflg) then
                 imrmn=-1
              else
                 imrmn=1
              end if
c
            do 127 imr=imrmn,imrmx,2
               if (mr.eq.0.and.nr.eq.0.and.ipnr.eq.0.and.
     #             ipar(lr+jr).ne.imr) go to 127
c
c----------------------------------------------------------------------
c     *** loop over iqnr ***
c----------------------------------------------------------------------
c
             iqnrmx=in2-iabs(ipnr)
             iqnrmn=-iqnrmx
             do 128 iqnr=iqnrmn,iqnrmx,2
c
c----------------------------------------------------------------------
c     *** loop over ipsr ***
c----------------------------------------------------------------------
c
              ipsrmx=1
              if (hfdflg) then
                 ipsrmn=-1
              else
                 ipsrmn=1
              end if
c
              do 130 ipsr=ipsrmn,ipsrmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqsr ***
c----------------------------------------------------------------------
c
               iqsrmx=1-iabs(ipsr)
               iqsrmn=-iqsrmx
               do 140 iqsr=iqsrmn,iqsrmx,2
c
                nrow=nrow+1
                if (nrow.le.mxdim) then
                  bss0(1,nrow)=lr
                  bss0(2,nrow)=mr
                  bss0(3,nrow)=kr
                  bss0(4,nrow)=jr
                  bss0(5,nrow)=nr
                  bss0(6,nrow)=ir
                  bss0(7,nrow)=ipnr
                  bss0(8,nrow)=imr
                  bss0(9,nrow)=iqnr
                  bss0(10,nrow)=ipsr
                  bss0(11,nrow)=iqsr
                end if
c
c----------------------------------------------------------------------
c     end loop over rows
c----------------------------------------------------------------------
c
 140           continue
 130          continue
 128         continue
 127        continue
 126       continue
 125       continue
 124      continue
 122     continue
 120    continue
 110   continue
 100  continue
c
c
      lthb=nrow
      return
c
 1001 format('*** Insufficient room for basis set ''',a,''' ***')
 1002 format('*** Error reading basis set ''',a,''' ***')
 1003 format('*** file ''',a,''' not found ***')
 1004 format('*** lbasix called with illegal MTS: (',7(i3,','),
     #       i3,') ***')
      end
