c**********************************************************************
c                    =========================
c                       subroutine STVECT
c                    =========================
c                      *** Pruning version ***
c
c       Calculates the starting vector for EPRSR
c       
c       Notes:
c               1) The orientational integral is evaluated numerically 
c                  using the Curtis-Clenshaw-Romberg algorithm.
c                  For details, see Bruno's thesis, p. 554.
c
c       written by AP 17-FEB-94, based on STVECT for EPRLL written
c       by DJS 10-SEP-87 - Version 1.4, modified so that nelv is kept
c       in common eprdat instead of passed as a subroutine argument.
c       DEB 22-OCT-91
c
c       Includes:
c               stddim.inc
c               eprdat.inc
c               indexl.inc
c               rndoff.inc
c
c       Uses:
c               ccrint.f
c               fact.f  (included in the code)
c
c**********************************************************************
c
      subroutine stvect(v,ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'srsset.inc'
      include 'rndoff.inc'
      include 'maxl.inc'
	include 'errmsg.inc'
c
      double precision v(2,MXDIM)
c
      integer ierr,nrow
      integer l,m,k,j,i,n,ipn,iqn,ips,iqs,im,ia,ik,ii,
     #                 ls,ms,ks,js,ns,is,
     #                 lbb,mb,kb,jb,nb,ib
      integer jq,jbmx,neval,id,iff,jj,nn,ipar,imxn,iberr
c
      logical check
c
      double precision stvt,vnorm,zero,fact,w3j,v0,dlm
c
      double precision acc,sml,bndlow,bndhi,ans,fct0,fct1
      parameter (acc=1.0D-8,sml=1.0D-10,zero=0.d0)
c
      double precision vst(0:mxlval),
     #     vtemp(0:mxlval1,-mxlval1:mxlval1,-mxlval1:mxlval1,
     #           0:mxlval1,-mxlval1:mxlval1),
     #     vmean(0:mxlval1,-mxlval1:mxlval1)
c
      double precision fzs
      external fzs
c
      common /ifdat/ iff,jj,nn,iberr
c
c######################################################################
c
c
c......................................................................
c
      jbmx=lemx+jmx
      bndlow=-1.d0
      bndhi=1.d0
c
c.......... store cage integrals  .....................................
c
      iff=0
      do jq=0,jbmx
         jj=jq
         nn=0
         call ccrint(bndlow,bndhi,acc,sml,ans,neval,fzs,id)
	   if (iberr.ne.0) ierr=BADBESS
         vst(jq)=ans
      end do
c
      nrow=0
      nelv=0
      vnorm=0.0D0
c
c.......... store interation integrals  ...............................
c
      bndlow=zero
      iff=1
c
      do l=0,lemx
      do m=-l,l
      do k=0,l
      do j=0,jmx
      if (dabs(bag).le.rndoff) then
        imxn=0
      else
        imxn=j
      end if
      do i=-imxn,imxn
c
      check=mod(l,2).eq.0.and.mod(k,2).eq.0.and.mod(j,2).eq.0
     #      .and.iabs(m).le.j.and.iabs(i).le.l
      if(check) then
c
           jj=l
           nn=k
           call ccrint(bndlow,bndhi,acc,sml,ans,neval,fzs,id)
	     if (iberr.ne.0) ierr=BADBESS
           fct0=fact(l-k)-fact(l+k)
           fct0=dexp(fct0/2)*ans
c
           if (k.ne.0) then
              fct0=2*fct0
           end if
c
c          if (i.ne.0) then
c             fct0=2*fct0
c          end if
c
           if (dabs(bag).le.rndoff) then
             dlm=1.0D0
           else
             call dlm0(dlm,l,i,bag)
           end if
           fct0=fct0*dsqrt(dble((2*l+1)*(2*j+1)))*ipar(i)*dlm
c
           v0=zero
           do jq=iabs(l-j),l+j
              fct1=(2*jq+1)*w3j(l,jq,j,-m,0,m)*w3j(l,jq,j,i,0,-i)
              v0=v0+fct1*vst(jq)
           end do
c
           stvt=fct0*v0
c
         else
           stvt=zero
         end if
c
      vtemp(l,m,k,j,i)=stvt
c
      end do
      end do
      end do
      end do
      end do
c
c.......... store probe integrals  ...............................
c
      iff=2
c
      do l=0,lemx
      do k=0,l
c
         check=mod(l,2).eq.0.and.mod(k,2).eq.0
c
         if (check) then
c
           jj=l
           nn=k
           call ccrint(bndlow,bndhi,acc,sml,ans,neval,fzs,id)
	     if (iberr.ne.0) ierr=BADBESS
           fct0=fact(l-k)-fact(l+k)
           fct0=dexp(fct0/2)*ans
c
           if (k.ne.0) then
              fct0=2*fct0
           end if
c
           fct0=fct0*dsqrt(dble(2*l+1))
c
           stvt=fct0
c
         else
           stvt=zero
         end if
c
         if (abs(stvt).lt.rndoff) then
           vmean(l,k)=zero
         else
           vmean(l,k)=stvt
         end if
c
      end do
      end do
c
c----------------------------------------------------------------------
c     evaluate starting vector
c----------------------------------------------------------------------
c
      do nrow=1,ndim
c
         l=bss0(1,nrow)
         m=bss0(2,nrow)
         k=bss0(3,nrow)
         j=bss0(4,nrow)
         n=bss0(5,nrow)
         i=bss0(6,nrow)
         ipn=bss0(7,nrow)
         im=bss0(8,nrow)
         iqn=bss0(9,nrow)
         ips=bss0(10,nrow)
         iqs=bss0(11,nrow)
c
c        if (k.eq.0) then
c           ik=ipar(l)
c        else
c           ik=1
c        end if
c
c        if (i.eq.0) then                                       !       ?
c           ii=ipar(j)
c        else
c           ii=1
c        end if
c
c        check=ipn.eq.0.and.iabs(ips).eq.1.and.
c    #   mod(l,2).eq.0.and.m.eq.-n.and.mod(k,2).eq.0.and.
c    #   iabs(k).le.l.and.ik.eq.1.and.im.eq.1
c    #   .and.iabs(i).le.l.and.iabs(i).le.j
c
         check=ipn.eq.0.and.mod(l,2).eq.0.and.mod(k,2).eq.0.and.m.eq.-n
c
         if (check) then
            stvt=zero
            do ls=0,lemx
            do ks=0,ls
               ms=0
               js=0
               ns=0
               is=0
               do lbb=0,lemx
                  kb=k-ks
                  mb=m
                  jb=j
                  nb=-mb
                  ib=i-is
                  fct0=dsqrt(dble((2*l+1)*(2*ls+1)*(2*lbb+1)))*
     #                 dsqrt(dble((2*j+1)*(2*js+1)*(2*jb+1)))*
     #                 ipar(m-k+n-i)*
     #                 w3j(l,ls,lbb,m,-ms,-mb)*w3j(l,ls,lbb,k,-ks,-kb)*
     #                 w3j(j,js,jb,n,-ns,-nb)*w3j(j,js,jb,i,-is,-ib)
                  stvt=stvt+fct0*vmean(ls,ks)*vtemp(lbb,mb,kb,jb,ib)
               end do
            end do
            end do
c           stvt=stvt*(ik+1)*(im+1)*(ii+1)
c           write (4,1000) l,m,k,j,n,i,ipn,stvt
         else
            stvt=zero
         end if

         v(1,nrow)=stvt
         vnorm=vnorm+stvt*stvt
c
c        if(abs(stvt).gt.rndoff) write(*,1000) l,m,k,j,n,i,ipn,stvt
      end do
 1000 format(2x,7(i2,','),f12.6)
c
c----------------------------------------------------------------------
c     normalize starting vector and zero out imaginary part
c----------------------------------------------------------------------
c
      nelv=0
      vnorm=1.0D0/dsqrt(vnorm)
      do 200 ia=1,ndim
        v(1,ia)=v(1,ia)*vnorm
        if(abs(v(1,ia)).gt.rndoff) then
          nelv=nelv+1
c         write(*,*) nelv,v(1,ia)
        else
          v(1,ia)=0.0D0
        end if
        v(2,ia)=0.0D0
 200  continue
c
c----------------------------------------------------------------------
c     zero out remainder of vector
c----------------------------------------------------------------------
c
      do 210 ia=ndim+1,MXDIM
        v(1,ia)=0.0D0
        v(2,ia)=0.0D0
 210  continue        
c
      return
      end
c
c**********************************************************************
c
c                       =========================
c                          subroutine DLM0
c                       =========================
c                                                             l
c       Calculate the reduced Wigner rotation matrix element d   (beta).
c                                                             m,0
c
c       written by ZL 28-Feb-95
c
c       Includes:
c               rndoff.inc
c               pidef.inc
c
c       Uses:
c
c**********************************************************************
c
c     subroutine dlm0(dlm,l,m,beta)
c
c     include 'pidef.inc'
c     include 'rndoff.inc'
c     include 'eprprm.inc'
c
c     integer l,m,mabs,mipar,ipar
c     double precision beta,dlm
c     double precision fct0,z,plgndr
c     external ipar,plgndr
c
c######################################################################
c
c     if(dabs(beta-90.0d0).lt.rndoff.or.
c    #   dabs(beta-270.0d0).lt.rndoff) then
c       z=0.0d0
c     else
c       z=dcos(beta*pi/180.0d0)
c     end if
c
c     if(m.lt.0) then
c       mipar=1
c     else
c       mipar=ipar(m)
c     end if
c
c     mabs=iabs(m)
c     fct0=fact(l-mabs)-fact(l+mabs)
c     fct0=dexp(fct0/2)*mipar
c
c     dlm=fct0*plgndr(l,mabs,z)
c
c     return
c     end
c
c
c       VERSION 1.0     12/17/88
c*********************************************************************
c
c                         FUNCTION FACT
c                         =============
c
c        This real function function returns the logarithm
c        of the factorial of the argument n (0 if n<=0)
c
c*********************************************************************
c
c     real function fact(n)
c
c     integer n
c
c     double precision zero
c     parameter (zero=0.d0)
c
c#####################################################################
c
c     if (n.le.1) then
c        fact=zero
c     else
c        fact=zero
c        do i=2,n
c           fact=fact+dlog(dble(i))
c        end do
c     end if
c
c     return
c     end
