
c**********************************************************************
c
c                       =================
c                       PROGRAM : BUILD 
c                       =================
c
c       This program reads in the index file <name>.ind 
c       written by EPRBS, and creates an output file containing a
c       list of basis vector indices determined by encreasing the
c       truncation values for quantum numbers relative to the SRLS,
c       but keeping track of the previously done pruning. Typically
c       one should start with a calculation with jmx=nmx=0,
c       no potential, calculate a large enough pruned basis set,
c       then run BUILD to add SRLS basis set, repeat pruning.
c
c       ... WARNING ... There is no warranty that the new basis
c                       set makes the spectrum to converge or
c                       to converge to the right result.
c
c       Includes:
c               stdio.inc
c               eprdat.inc
c               stddim.inc
c               fnames.inc
c               indexl.inc
c               rndoff.inc
c
c       Uses:
c               ipar.f
c               getids.f
c               setnam.f
c               setflg.f
c               lbasix.f
c               rddat.f
c
c**********************************************************************
c
      program build
c
      implicit none
c
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'stddim.inc'
      include 'fnames.inc'
      include 'indexl.inc'
      include 'rndoff.inc'
c
      integer i,j,ierr,ifile,nbss,nfiles
      double precision basis(2,mxdim),bmax
      logical fexist
c
c
c######################################################################
c
      write(luttyo,1000)
      write(luttyo,1010)
c
      ifile=0
      call getids(nfiles,mxcalc)
      if (nfiles.eq.0) goto 9999
c
 1    ifile=ifile+1
      if (ifile.gt.nfiles) goto 9999
      fileid=files(ifile)
      call setnam
      call setflg( flags(i) )
c
c----------------------------------------------------------------------
c     read input parameters from input files
c----------------------------------------------------------------------
c
        inquire(file=prname,exist=fexist)
c
        if (fexist) then
          call rddat(prname,namlth,ierr)
          if (ierr.eq.-100) then
            write (luttyo,1201) prname(:namlth)
            go to 1
          else if (ierr.ne.0) then
            write (luttyo,1200) prname(:namlth)
            go to 1
          end if
        else
          write (luttyo,1100) prname(:namlth)
          go to 1
        end if
c
c----------------------------------------------------------------------
c     Build the full list of basis indices corresponding to the MTS
c----------------------------------------------------------------------
c
      call srsbld( ixname,namlth,0 )
c
c----------------------------------------------------------------------     
c   Write out basis set indices for pruned set
c----------------------------------------------------------------------
c
      call wrdat(prname,namlth)
c
      write (luttyo,5010) ixname(:namlth)
      open (unit=ludisk,file=ixname(:namlth),status='unknown',
     #     access='sequential',form='formatted')
      write (ludisk,6010) ndim,lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
      write (ludisk,6020)
      write (ludisk,6030) (l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),
     #               pi1(i),qi1(i),ps1(i),qs1(i),i=1,ndim)
      close (unit=ludisk)
      go to 1
c
c----------------------------------------------------------------------
c     exit from program
c----------------------------------------------------------------------
c
 9999 write(luttyo,1000)
      stop
c
c======================================================================
c    format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(28x,'Program : BUILD',/,28x,'--------------',/)
c
 1100 format(/,20x,'*** Cannot find file ',a,' ****',/)
 1200 format(/,10x,'*** Error reading file ',a,' ***',/)
 1201 format(/,10x,'*** ''',a,''' is not an EPRSR file',
     #' ***')
 5000 format(15x,38('='),/,
     #     15x,'Pruning procedure ',i2,' using file ',a,/,
     #     15x,38('='),/)
 5010 format(/,20x,'*** writing file ',a,' ***')
 5020 format(20x,'MTS basis dimension: ',i4/
     #       20x,'pruned dimension:    ',i4/
     #       20x,'pruning tolerance :  ',f8.4/)
 6010 format(i6,8(i4))
 6020 format(/,2x,76('-')/
     #       3x,'Lo',3x,'Mo',3x,'Ko',3x,
     #          'Lc',3x,'Mc',3x,'Kc',3x,
     #          'pI',3x,'qI',3x,'pS',3x,'qS')
 6030 format(10(i5))
c
c======================================================================
c
      end

c
c**********************************************************************
c                       ===================
c                       SUBROUTINE : SRSBLD
c                       ===================
c
c       This routine builds a list of the basis set indices.
c       Basically it is lbasix.f - Version 1.4. It reads <name>.ind
c       and it creates a new basis set with all the possible pruned
c       for quantum numbers other than jmx and nmx.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexl.inc
c               srsflg.inc
c
c       Uses:
c               ipar.f
c
c######################################################################
c
c See accompanying text for complete and accurate definition of
c the symmetrized basis set. Note that
c
c mr .ge. 0
c kr .ge. 0
c 
c Particular cases mr=nr=ipnr=0   --->  imr=ipar(lr+jr)
c                  mr=nr=0        --->  ipnr .ge. 0
c                  mr=0           --->  nr   .ge. 0
c
c######################################################################
c
c**********************************************************************
c
      subroutine srsbld( ixname,lth,new )
c
      implicit none
      character*30 ixname
      integer lth,new,i
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexl.inc'
      include 'srsexl.inc'
      include 'srsflg.inc'
c
      integer lr,mr,mrmx,kr,krmx,
     #        jr,nr,nrmn,nrmx,ir,irmx,
     #        ipsr,iqsr,ipnr,iqnr,
     #        ipnrmn,ipnrmx,iqnrmn,iqnrmx,
     #        ipsrmn,ipsrmx,iqsrmn,iqsrmx,
     #        imr,imrmn,imrmx,imstep,
     #        nrow
c
      logical fexist
      integer ipar
      external ipar
c
      character*80 line
      integer itoken
      logical fprmpt,prmpt,check
      external prmpt,itoken
c
c######################################################################
c
      if (new.eq.0) then
c
c----------------------------------------------------------------------
c     Check whether the basis set indices may be found in a file
c----------------------------------------------------------------------
c
         inquire(file=ixname(:lth),exist=fexist)
         if (fexist) then
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
            open (unit=ludisk,file=ixname(:lth),status='old',
     #           access='sequential',form='unformatted',err=9)
            read (ludisk,err=9) ndim,lemx,lomx,mmx,kmx,
     #                          jmx,nmx,imx,ipnmx
            read (ludisk,err=9) (l1(i),m1(i),k1(i),j1(i),n1(i),
     #            i1(i),pi1(i),im1(i),qi1(i),ps1(i),qs1(i),
     #            i=1,ndim)
            close (unit=ludisk)
            write (luttyo,3000) ndim
            go to 200
          end if
c
c Error: close file and stop
c
 9        close (unit=ludisk)
          write (luttyo,4000) ixname(:lth)
          stop
c
      end if
c
 2    write (luttyo,2000) jmx,nmx
      fprmpt=prmpt( line )
      if (fprmpt) then
         jmx=itoken( line )
         nmx=itoken( line )
      end if
c
c######################################################################
c
      nrow=0
c
c----------------------------------------------------------------------
c     *** loop over lr ***
c----------------------------------------------------------------------
c
      do 100 lr=0,lemx,1
        if((1-2*mod(abs(lr),2).ne.1).and.(lr.gt.lomx)) go to 100
c
c----------------------------------------------------------------------
c     *** loop over mr ***
c----------------------------------------------------------------------
c
       mrmx=min(mmx,lr)
       do 110 mr=0,mrmx,1
c
c----------------------------------------------------------------------
c     *** loop over kr ***
c----------------------------------------------------------------------
c
        krmx=min(kmx,lr)
        do 120 kr=0,krmx,kdelta
c 
c----------------------------------------------------------------------
c     *** loop over jr ***
c----------------------------------------------------------------------
c
         do 122 jr=0,jmx,jdelta
c
c----------------------------------------------------------------------
c     *** loop over nr ***
c----------------------------------------------------------------------
c
          nrmx=min(nmx,jr)
c
          if (mr.eq.0) then
             nrmn=0
          else
             nrmn=-nrmx
          end if
          do 124 nr=nrmn,nrmx,1
c
c----------------------------------------------------------------------
c     *** loop over ir ***
c----------------------------------------------------------------------
c
          irmx=min(imx,jr)
          do 125 ir=0,irmx
c
c----------------------------------------------------------------------
c     *** loop over ipnr ***
c----------------------------------------------------------------------
c
           ipnrmx=min(ipnmx,in2)
           if (mr.eq.0.and.nr.eq.0) then
              ipnrmn=0
           else
              ipnrmn=-ipnrmx
           end if
           do 126 ipnr=ipnrmn,ipnrmx,1
c
c----------------------------------------------------------------------
c     *** loop over im ***
c----------------------------------------------------------------------
c
              if (mr.eq.0.and.nr.eq.0.and.ipnr.eq.0) then
                imrmn=ipar(lr+jr)
                imrmx=imrmn
              else
                imrmx=1
                if (hfdflg) then
                  imrmn=-1
                else
                  imrmn=1
                end if
              end if
c
            do 127 imr=imrmn,imrmx,2
c
c----------------------------------------------------------------------
c     *** loop over iqnr ***
c----------------------------------------------------------------------
c
             iqnrmx=in2-iabs(ipnr)
             iqnrmn=-iqnrmx
             do 128 iqnr=iqnrmn,iqnrmx,2
c
c----------------------------------------------------------------------
c     *** loop over ipsr ***
c----------------------------------------------------------------------
c
              ipsrmx=1
              if (hfdflg) then
                 ipsrmn=-1
              else
                 ipsrmn=1
              end if
c
              do 130 ipsr=ipsrmn,ipsrmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqsr ***
c----------------------------------------------------------------------
c
               iqsrmx=1-iabs(ipsr)
               iqsrmn=-iqsrmx
               do 140 iqsr=iqsrmn,iqsrmx,2
c
c       write (*,900) lr,mr,kr,ipnr,imr,iqnr,ipsr,iqsr
c                    pause
c
                  check=.false.
                  do i=1,ndim
                    check=l1s(i).eq.lr.and.m1s(i).eq.mr.and.
     #              k1s(i).eq.kr.and.pi1s(i).eq.ipnr.and.
     #              im1s(i).eq.imr.and.qi1s(i).eq.iqnr.and.
     #              ps1s(i).eq.ipsr.and.qs1s(i).eq.iqsr
c
c      write (*,901) l1s(i),m1s(i),k1s(i),pi1s(i),im1s(i),qi1s(i),
c     #ps1s(i),qs1s(i)
c 900  format(2x,///,50('-'),/2x,8(i2,','),/,2x,50('-'))
c 901  format(2x,8(i2,','))
c
                    if (check) then
                    nrow=nrow+1
                    if (nrow.le.mxdim) then
                      l1(nrow)=lr
                      m1(nrow)=mr
                      k1(nrow)=kr
                      j1(nrow)=jr
                      n1(nrow)=nr
                      i1(nrow)=ir
                      pi1(nrow)=ipnr
                      im1(nrow)=imr
                      qi1(nrow)=iqnr
                      ps1(nrow)=ipsr
                      qs1(nrow)=iqsr
                    end if
                    go to 1
                  end if
                end do
c
 1              continue
c
c----------------------------------------------------------------------
c     end loop over rows                                      
c----------------------------------------------------------------------
c
 140           continue
 130          continue
 128         continue
 127        continue
 126       continue
 125       continue
 124      continue
 122     continue
 120    continue
 110   continue
 100  continue
c
      ndim=nrow
      write (luttyo,3000) ndim
 200  return
c
 2000 format (2x,'truncation values   [jmx,nmx] : ',2(i2,2x))
 3000 format (/,20x,'+++ ndim = ',i7,' +++',/)
 4000 format(/,20x,'*** Cannot find file ',a,' ****',/)
c
      end







