c**********************************************************************
c                    =========================
c                       subroutine STVECT
c                    =========================
c                      *** Pruning version ***
c
c       Calculates the starting vector for EPRSR
c       
c       Notes:
c               1) The orientational integral is evaluated numerically 
c                  using the Curtis-Clenshaw-Romberg algorithm.
c                  For details, see Bruno's thesis, p. 554.
c
c       written by AP 17-FEB-94, based on STVECT for EPRLL written
c       by DJS 10-SEP-87 - Version 1.4, modified so that nelv is kept
c       in common eprdat instead of passed as a subroutine argument.
c       DEB 22-OCT-91
c
c       Includes:
c               stddim.inc
c               eprdat.inc
c               indexl.inc
c               rndoff.inc
c
c       Uses:
c               ccrint.f
c               fact.f  (included in the code)
c
c**********************************************************************
c
      subroutine pstves(bss,v)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'rndoff.inc'
      include 'maxl.inc'
c
      integer bss(10,MXDIM)
      double precision v(2,MXDIM)
c
      integer nrow
      integer l,m,k,j,n,ipn,iqn,ips,iqs,im,i,ik
      integer jq,jbmx,neval,id,iff,jj,nn,ipar
c
      logical check
c
      double precision stvt,vnorm,zero,fact,w3j,v0
c
      double precision acc,sml,bndlow,bndhi,ans,fct0,fct1
      parameter (acc=1.0D-8,sml=1.0D-10,zero=0.d0)
c
      double precision vst(mxlval+1)
c
      double precision fzs
      external fzs
c
      common /ifdat/ iff,jj,nn
c
c######################################################################
c
c
c......................................................................
c
      jbmx=2*mxlval
      bndlow=-1.d0
      bndhi=1.d0
c
c.......... store SRLS integrals  .....................................
c
      iff=2
      do jq=0,jbmx
         jj=jq
         nn=0
         call ccrint(bndlow,bndhi,acc,sml,ans,neval,fzs,id)
         vst(jq+1)=ans
      end do
c
      nrow=0
      nelv=0
      vnorm=0.0D0
c
c----------------------------------------------------------------------
c     *** loop over rows ***
c----------------------------------------------------------------------
c
      bndlow=zero
      iff=1
c
      do nrow=1,ndim
c
         l=bss(1,nrow)
         m=bss(2,nrow)
         k=bss(3,nrow)
         j=bss(4,nrow)
         n=bss(5,nrow)
         ipn=bss(6,nrow)
         im=bss(7,nrow)
         iqn=bss(8,nrow)
         ips=bss(9,nrow)
         iqs=bss(10,nrow)
c
         if (k.eq.0) then
            ik=ipar(l)
         else
            ik=1
         end if
c
         check=ipn.eq.0.and.iabs(ips).eq.1.and.
     #   mod(l,2).eq.0.and.m.eq.-n.and.mod(k,2).eq.0.and.
     #   iabs(k).le.l.and.ik.eq.1.and.im.eq.1
c
         if (check) then
c
           jj=l
           nn=k
           call ccrint(bndlow,bndhi,acc,sml,ans,neval,fzs,id)
           fct0=fact(l-k)-fact(l+k)
           fct0=dexp(fct0/2)*ans
c
           if (k.ne.0) then
              fct0=2*fct0
           end if
c
           fct0=fct0*dsqrt(dble((2*l+1)*(2*j+1)))
c
           v0=zero
           do jq=iabs(l-j),l+j
              fct1=(2*jq+1)*w3j(l,jq,j,-m,0,m)*w3j(l,jq,j,0,0,0)
              v0=v0+fct1*vst(jq+1)
           end do
c
           stvt=fct0*v0
c
         else
           stvt=zero
         end if
c
         v(1,nrow)=stvt
         vnorm=vnorm+stvt*stvt
c
      end do
c
c----------------------------------------------------------------------
c     normalize starting vector and zero out imaginary part
c----------------------------------------------------------------------
c
      nelv=0
      vnorm=1.0D0/dsqrt(vnorm)
      do 200 i=1,ndim
        v(1,i)=v(1,i)*vnorm
        if(abs(v(1,i)).gt.rndoff) then
          nelv=nelv+1
        else
          v(1,i)=0.0D0
        end if
        v(2,i)=0.0D0
 200  continue
c
c----------------------------------------------------------------------
c     zero out remainder of vector
c----------------------------------------------------------------------
c
      do 210 i=ndim+1,MXDIM
        v(1,i)=0.0D0
        v(2,i)=0.0D0
 210  continue        
c
      return
      end















c       VERSION 1.0     12/17/88
c*********************************************************************
c
c                         FUNCTION FACT
c                         =============
c
c        This real function function returns the logarithm
c        of the factorial of the argument n (0 if n<=0)
c
c*********************************************************************
c
      real function fact(n)
c
      integer n
c
      double precision zero
      parameter (zero=0.d0)
c
c#####################################################################
c
      if (n.le.1) then
         fact=zero
      else
         fact=zero
         do i=2,n
            fact=fact+dlog(dble(i))
         end do
      end if
c
      return
      end
